@@ -28,8 +28,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
-
-#include <dbus/dbus-glib.h>
+#include <gio/gio.h>
 
 #include "gsm-util.h"
 
@@ -234,9 +233,9 @@ gsm_util_get_app_dirs ()
         dirs = g_ptr_array_new ();
 
         g_ptr_array_add (dirs,
-			 g_build_filename (g_get_user_data_dir (),
-					   "applications",
-					   NULL));
+                         g_build_filename (g_get_user_data_dir (),
+                                           "applications",
+                                           NULL));
 
         system_data_dirs = g_get_system_data_dirs ();
         for (i = 0; system_data_dirs[i]; i++) {
@@ -255,15 +254,15 @@ char **
 gsm_util_get_desktop_dirs (gboolean include_saved_session,
                            gboolean autostart_first)
 {
-	char **apps;
-	char **autostart;
-	char **standard_autostart;
-	char **result;
-	int    size;
-	int    i;
+        char **apps;
+        char **autostart;
+        char **standard_autostart;
+        char **result;
+        int    size;
+        int    i;
 
-	apps = gsm_util_get_app_dirs ();
-	autostart = gsm_util_get_autostart_dirs ();
+        apps = gsm_util_get_app_dirs ();
+        autostart = gsm_util_get_autostart_dirs ();
 
         /* Still, check the standard autostart dirs for things like fulfilling session reqs,
          * if using a non-standard autostart dir for autostarting */
@@ -272,15 +271,15 @@ gsm_util_get_desktop_dirs (gboolean include_saved_session,
         else
                 standard_autostart = NULL;
 
-	size = 0;
-	for (i = 0; apps[i] != NULL; i++) { size++; }
-	for (i = 0; autostart[i] != NULL; i++) { size++; }
+        size = 0;
+        for (i = 0; apps[i] != NULL; i++) { size++; }
+        for (i = 0; autostart[i] != NULL; i++) { size++; }
         if (autostart_dirs != NULL)
                 for (i = 0; standard_autostart[i] != NULL; i++) { size++; }
         if (include_saved_session)
                 size += 1;
 
-	result = g_new (char *, size + 1); /* including last NULL */
+        result = g_new (char *, size + 1); /* including last NULL */
 
         size = 0;
 
@@ -316,13 +315,13 @@ gsm_util_get_desktop_dirs (gboolean include_saved_session,
                         result[size++] = g_strdup (gsm_util_get_saved_session_dir ());
         }
 
-	g_free (apps);
-	g_free (autostart);
-	g_free (standard_autostart);
+        g_free (apps);
+        g_free (autostart);
+        g_free (standard_autostart);
 
-	result[size] = NULL;
+        result[size] = NULL;
 
-	return result;
+        return result;
 }
 
 gboolean
@@ -448,56 +447,79 @@ gsm_util_generate_startup_id (void)
                                 sequence);
 }
 
+static void
+gsm_util_save_environment (void)
+{
+        g_autoptr(GVariant)  environment = NULL;
+        g_autoptr(GError)    error = NULL;
+        g_autofree char     *environment_dir = NULL;
+        g_autofree char     *filename = NULL;
+        gconstpointer        environment_data;
+        gsize                size;
+        int                  ret;
+        gboolean             contents_set;
+
+        environment_dir = g_build_filename (g_get_user_runtime_dir (), "gnome", NULL);
+        ret = g_mkdir_with_parents (environment_dir, 0700);
+
+        if (ret < 0) {
+                g_warning ("can't create gnome runtime dir: %m");
+                return;
+        }
+
+        environment = g_variant_new_bytestring_array ((const gchar * const *) g_get_environ (), -1);
+        size = g_variant_get_size (environment);
+        environment_data = g_variant_get_data (environment);
+
+        filename = g_build_filename (environment_dir, "environment", NULL);
+
+        contents_set = g_file_set_contents (filename, environment_data, size, &error);
+
+        if (!contents_set) {
+                g_warning ("can't save environment: %s", error->message);
+        }
+}
+
 static gboolean
 gsm_util_update_activation_environment (const char  *variable,
                                         const char  *value,
                                         GError     **error)
 {
-        DBusGConnection *dbus_connection;
-        DBusGProxy      *bus_proxy;
-        GHashTable      *environment;
+        GDBusConnection *connection;
         gboolean         environment_updated;
+        GVariantBuilder  builder;
+        GVariant        *reply;
+        GError          *bus_error = NULL;
 
         environment_updated = FALSE;
-        bus_proxy = NULL;
-        environment = NULL;
+        connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, error);
 
-        dbus_connection = dbus_g_bus_get (DBUS_BUS_SESSION, error);
-
-        if (dbus_connection == NULL) {
+        if (connection == NULL) {
                 return FALSE;
         }
 
-        bus_proxy = dbus_g_proxy_new_for_name_owner (dbus_connection,
-                                                     DBUS_SERVICE_DBUS,
-                                                     DBUS_PATH_DBUS,
-                                                     DBUS_INTERFACE_DBUS,
-                                                     error);
+        g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{ss}"));
+        g_variant_builder_add (&builder, "{ss}", variable, value);
 
-        if (bus_proxy == NULL) {
-                goto out;
+        reply = g_dbus_connection_call_sync (connection,
+                                             "org.freedesktop.DBus",
+                                             "/org/freedesktop/DBus",
+                                             "org.freedesktop.DBus",
+                                             "UpdateActivationEnvironment",
+                                             g_variant_new ("(@a{ss})",
+                                                            g_variant_builder_end (&builder)),
+                                             NULL,
+                                             G_DBUS_CALL_FLAGS_NONE,
+                                             -1, NULL, &bus_error);
+
+        if (bus_error != NULL) {
+                g_propagate_error (error, bus_error);
+        } else {
+                environment_updated = TRUE;
+                g_variant_unref (reply);
         }
 
-        environment = g_hash_table_new (g_str_hash, g_str_equal);
-
-        g_hash_table_insert (environment, (void *) variable, (void *) value);
-
-        if (!dbus_g_proxy_call (bus_proxy,
-                                "UpdateActivationEnvironment", error,
-                                DBUS_TYPE_G_STRING_STRING_HASHTABLE,
-                                environment, G_TYPE_INVALID,
-                                G_TYPE_INVALID))
-                goto out;
-
-        environment_updated = TRUE;
-
- out:
-
-        g_clear_object (&bus_proxy);
-
-        if (environment != NULL) {
-                g_hash_table_destroy (environment);
-        }
+        g_clear_object (&connection);
 
         return environment_updated;
 }
@@ -522,4 +544,6 @@ gsm_util_setenv (const char *variable,
                 g_warning ("Could not make bus activated clients aware of %s=%s environment variable: %s", variable, value, bus_error->message);
                 g_error_free (bus_error);
         }
+
+        gsm_util_save_environment ();
 }
